import boto3
import click
from pool_map import multi_process
import logging
from termcolor import colored
from botocore.exceptions import ClientError

public_perms = "http://acs.amazonaws.com/groups/global/AllUsers"


def get_bucket_permission(bucket):
    s3 = boto3.resource( 's3' )
    bkt = s3.Bucket( bucket )
    bucket_acl = bkt.Acl().grants
    owner = bkt.Acl().owner['ID']
    perms = {'Public': [], 'Canonical': [], 'Owner': []}
    print("Owner ID: %s" %owner)
    for p in bucket_acl:
        if p['Grantee']['Type'] == "Group":
            if p['Grantee']['URI'] == public_perms:
                perms['Public'].append(p['Permission'])
        if p['Grantee']['Type'] == "CanonicalUser":
            o = p['Grantee']['ID']
            if owner != o:
                perms['Canonical'].append(p['Permission'])
            if owner == o:
                perms['Owner'].append( p['Permission'] )
    if perms['Public']:
        print(colored('Public Access: %s', 'red')) % perms['Public']
    if perms['Canonical']:
        print(colored('Canonical Access: %s', 'yellow')) % perms['Canonical']
    print(colored('Bucket Owner: %s', 'green')) % perms['Owner']


def get_permission(x):
    s3 = boto3.resource( 's3' )
    bucket, key = x[0], x[1]
    bkt = s3.Bucket( bucket )
    owner = bkt.Acl().owner['ID']
    obj = s3.Object(bucket, key)
    failed = False
    try:
        obj_acl = obj.Acl().grants
    except ClientError as e:
        if e.response['Error']['Message'] == "Access Denied":
            click.echo('Failed to retrieve ACLs for %s with error Access Denied' %key)
            failed = True
            pass
    perms = {'Public': [], 'Canonical': [], 'Owner': []}
    if not failed:
        for p in obj_acl:
            if p['Grantee']['Type'] == "Group":
                if p['Grantee']['URI'] == public_perms:
                    perms['Public'].append( p['Permission'] )
            if p['Grantee']['Type'] == "CanonicalUser":
                o = p['Grantee']['ID']
                if owner != o:
                    perms['Canonical'].append(p['Permission'])
                if owner == o:
                    perms['Owner'].append( p['Permission'] )

        if perms['Public'] or perms['Canonical']:
            print('KEY: %s ; Permissions: %s' % (key, perms))
        else:
            logging.warning('Key is Safe: %s' % key)


def scan_key_perms(scanperms, bucket, prefix, workers):
    s3 = boto3.resource( 's3' )
    bkt = s3.Bucket( bucket )
    click.echo('Scanning bucket permissions')
    # get bucket permissions first
    get_bucket_permission(bucket)
    click.echo(30*'=')
    click.echo('Scanning object permissions')
    objects = []
    iterator = bkt.objects.filter( Prefix=prefix )
    processed = False
    keys_processed = 0
    for k in iterator:
        keys_processed += 1
        processed = False
        if len(objects) < 1000:
            objects.append( [bucket, k.key] )
        else:
            multi_process(get_permission, objects, workers)
            del objects[:]
            processed = True
    if not processed:
        multi_process(get_permission, objects, workers)

    click.echo('{} keys processed'.format(keys_processed))


import click
from s3_restore import *
from s3_empty_bucket import *
from s3_scan_key_perms import *
from __init__ import *

@click.group()
def cli():
    """This is your s3 friend(bro), that will help you with bucket iterations.
    Try:\n
    # s3bro restore --help\n
    # s3bro purge --help\n
    # s3tk --help\n
    """
    pass


def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()


@cli.command()
@click.argument('restore', nargs=-1)
@click.option('--bucket', type=str, help='bucket name', required=True)
@click.option('--prefix', type=str, help=' prefix', default='')
@click.option('--days', type=int, help='Days to keep the restore', required=True)
@click.option('--type', type=click.Choice(['Standard', 'Expedited', 'Bulk']), help='restore type (Tier)', required=True)
@click.option('--versions/--no-versions', default=False, help='[--no-versions is DEFAULT] - this option will make the restore to include all versions excluding delete markers')
@click.option('--update-restore-date/--do-not-update-restore-date', default=False, help='If passed, it will change the restore date for already restored key')
@click.option('--include', type=str, multiple=True, help='Only restore keys that matches with a given'
                                  ' string, you can add multiples times by passing --include multiple times')
@click.option('--exclude', type=str, multiple=True, help='Do not restore if key name matches with a given pattern,'
                                  'you can add multiple patterns by inputting')
@click.option('--workers', type=int, help='How many helpers to include in task, default is 10', default=10)
@click.option('--log-level', type=click.Choice(['INFO', 'ERROR', 'DEBUG', 'WARNING']), help='logging type', default='ERROR')
def restore(restore, bucket, prefix, days, type, versions, update_restore_date, workers, include, exclude, log_level):
    loglevel(log_level)
    collect_keys(restore, bucket, prefix, days, type, versions, update_restore_date, workers, include, exclude)


@cli.command()
@click.argument('purge', nargs=-1)
@click.option('--bucket', type=str, help='Bucket name', required=True)
@click.option('--prefix', type=str, default='', help='prefix name - optional')
@click.option('--yes', is_flag=True, callback=abort_if_false,
              expose_value=False,
              prompt='Are you sure you want to wipe the bucket?', help="first confirmation")
@click.option('--yes-really', is_flag=True, callback=abort_if_false,
              expose_value=False,
              prompt="I'm going to delete everything in your bucket, are you really sure?", help="second confirmation")
@click.option('--log-level', type=click.Choice(['INFO', 'ERROR', 'DEBUG', 'WARNING']), help='logging type', default='ERROR')
def purge(purge, bucket, prefix, log_level):
    loglevel(log_level)
    clean_bucket(bucket, prefix)


@cli.command()
@click.argument('scanperms', nargs=-1)
@click.option('--bucket', type=str, help='Bucket name', required=True)
@click.option('--prefix', type=str, default='', help='prefix name - optional')
@click.option('--workers', type=int, help='How many helpers to include in task, default is 10', default=10)
@click.option('--log-level', type=click.Choice(['INFO', 'ERROR', 'DEBUG', 'WARNING']), help='logging type', default='ERROR')
def scanperms(scanperms, bucket, prefix, workers, log_level):
    loglevel(log_level)
    scan_key_perms(scanperms, bucket, prefix, workers)

